using System;
using System.Data;
using System.Configuration;
using System.Net;
using System.IO;
using System.Security.Cryptography;
using System.Text;
using System.Threading;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

/// <summary>
/// Summary description for UrlCacheDependency
/// reference: http://www.developer.com/net/article.php/3371531
/// </summary>
public class UrlCacheDependency : System.Web.Caching.CacheDependency
{
    protected DateTime lastModified;
    protected DateTime ifModifiedSince;
    protected HttpStatusCode statusCode;
    protected byte[] contents;
    protected long contentLength;
    protected byte[] hashCode;
    protected WebRequest request;
    protected StreamReader reader;
    protected HttpWebResponse response;
    protected System.Threading.Timer timer;
    protected MD5CryptoServiceProvider hashProvider;

    #region Properties
    private TimeSpan pollFrequency = new TimeSpan(0, 5, 0);
    public TimeSpan PollFrequency
    {
        get { return pollFrequency; }
        set { pollFrequency = value; }
    }
    private string url = "";
    public string Url
    {
        get { return url; }
        set { url = value; }
    }
    #endregion

    #region Constructors
    public UrlCacheDependency(string url)
	{
        this.Url = url;
        this.Initialize();
	}

    public UrlCacheDependency(string url, TimeSpan pollFrequency)
    {
        this.Url = url;
        this.PollFrequency = pollFrequency;
        this.Initialize();
    }
    #endregion

    protected void Initialize()
    {
        try
        {
            hashProvider = new MD5CryptoServiceProvider();
            ifModifiedSince = DateTime.UtcNow;
            GetInitialUrl();
            TimerCallback callback = new TimerCallback(CheckUrlForChanges);
            timer = new Timer(callback, null, this.PollFrequency, this.PollFrequency);
        }
        catch { }
        finally {
        }
    }

    protected void GetInitialUrl()
    {
        try
        {
            request = WebRequest.Create(url);
            request.Method = "GET";
            response = (HttpWebResponse)request.GetResponse();

            // store info about the response
            lastModified = response.LastModified;
            statusCode = response.StatusCode;

            if(statusCode != HttpStatusCode.OK)
            {
                NotifyDependencyChanged(this, new EventArgs());
            }

            reader = new StreamReader(response.GetResponseStream());
            // check response.ContentEncoding to determine encoding type
            contents = Encoding.UTF8.GetBytes(reader.ReadToEnd());
            contentLength = contents.LongLength;

            hashCode = hashProvider.ComputeHash(contents);

        }
        catch {
            base.NotifyDependencyChanged(this, new EventArgs());
            throw;
        }
        finally {
            if(reader!=null)
                reader.Close();
        }
    }

    
    protected void CheckUrlForChanges(object arg)
    {

        DateTime newLastModified;
        byte[] newContents;
        byte[] newHashCode;
        long newContentLength;

        request = WebRequest.Create(url);
        request.Method = "GET";
        response = (HttpWebResponse)request.GetResponse();

        newLastModified = response.LastModified;        
        if(newLastModified > lastModified)
        {
            base.NotifyDependencyChanged(this, new EventArgs());
        }

        try
        {
            reader = new StreamReader(response.GetResponseStream());
            // check response.ContentEncoding to determine encoding type
            newContents = Encoding.UTF8.GetBytes(reader.ReadToEnd());
            newContentLength = newContents.LongLength;
            newHashCode = hashProvider.ComputeHash(newContents);

            if ((newContentLength != contentLength) ||
                (newContents != contents) ||
                (newHashCode != hashCode))
            {
                base.NotifyDependencyChanged(this, new EventArgs());
            }
        }
        catch { }
        finally
        {
            if (reader != null)
            {
                reader.Close();
            }
        }
    }
}
